import { defineComponent, ref, onActivated, onDeactivated, onMounted, onBeforeUnmount, onUnmounted, pushScopeId, popScopeId, resolveComponent, openBlock, createBlock, Fragment, createVNode, renderList, withScopeId } from 'vue';

const lerp = (current, goal, p) => (1 - p) * current + p * goal;
const flipVal = (val, min, max) => Math.abs(val * (Math.sign(val) || 1) - max) + min;
function valToP(value, min, max) {
  if (min > max) {
    [min, max] = [max, min];
    value = flipVal(value, min, max);
  }

  return (value - min) / (max - min);
}

/*!
  * vue-global-events v2.0.1
  * (c) 2020 Damian Dulisz, Eduardo San Martin Morote
  * @license MIT
  */

let _isIE;
function isIE() {
    return _isIE == null
        ? (_isIE = /msie|trident/.test(window.navigator.userAgent.toLowerCase()))
        : _isIE;
}

const EVENT_NAME_RE = /^on(\w+?)((?:Once|Capture|Passive)*)$/;
const MODIFIERS_SEPARATOR_RE = /[OCP]/g;
function extractEventOptions(modifiersRaw) {
    if (!modifiersRaw)
        return;
    // IE only supports capture option and it has to be a boolean
    // https://github.com/shentao/vue-global-events/issues/14
    if (isIE()) {
        return modifiersRaw.includes('Capture');
    }
    const modifiers = modifiersRaw
        .replace(MODIFIERS_SEPARATOR_RE, ',$&')
        .toLowerCase()
        // remove the initial comma
        .slice(1)
        .split(',');
    return modifiers.reduce((options, modifier) => {
        options[modifier] = true;
        return options;
    }, {});
}
const GlobalEventsImpl = defineComponent({
    name: 'GlobalEvents',
    props: {
        target: {
            type: String,
            default: 'document',
        },
        filter: {
            type: Function,
            default: () => () => true,
        },
    },
    setup(props, { attrs }) {
        let activeListeners = Object.create(null);
        const isActive = ref(true);
        onActivated(() => {
            isActive.value = true;
        });
        onDeactivated(() => {
            isActive.value = false;
        });
        onMounted(() => {
            Object.keys(attrs)
                .filter((name) => name.startsWith('on'))
                .forEach((eventNameWithModifiers) => {
                const listener = attrs[eventNameWithModifiers];
                const listeners = Array.isArray(listener) ? listener : [listener];
                const match = eventNameWithModifiers.match(EVENT_NAME_RE);
                if (!match) {
                    return;
                }
                let [, eventName, modifiersRaw] = match;
                eventName = eventName.toLowerCase();
                const handlers = listeners.map((listener) => (event) => {
                    isActive.value &&
                        props.filter(event, listener, eventName) &&
                        listener(event);
                });
                const options = extractEventOptions(modifiersRaw);
                handlers.forEach((handler) => {
                    window[props.target].addEventListener(eventName, handler, options);
                });
                activeListeners[eventNameWithModifiers] = [
                    handlers,
                    eventName,
                    options,
                ];
            });
        });
        onBeforeUnmount(() => {
            for (const eventNameWithModifiers in activeListeners) {
                const [handlers, eventName, options] = activeListeners[eventNameWithModifiers];
                handlers.forEach((handler) => {
                    window[props.target].removeEventListener(eventName, handler, options);
                });
            }
            activeListeners = {};
        });
        return () => null;
    },
});
// export the public type for h/tsx inference
// also to avoid inline import() in generated d.ts files
/**
 * Component of vue-lib.
 */
const GlobalEvents = GlobalEventsImpl;

const setTransform = (element, x, y) => {
  if (element.style) element.style.transform = `translateX(${x}px) translateY(${y}px)`;
};

const followMouse = (element, p, target) => {
  const {
    left,
    top
  } = element.getBoundingClientRect(); // Apply LERP

  const x = lerp(left, target.x, p);
  const y = lerp(top, target.y, p); // Set Position using Transforms

  setTransform(element, x, y);
};

var script = defineComponent({
  name: 'GooeyCursor',
  components: {
    GlobalEvents
  },
  props: {
    size: {
      type: Number,
      default: 32
    },
    length: {
      type: Number,
      default: 8
    },
    color: {
      type: String,
      default: undefined
    },
    hover: {
      type: String,
      default: 'a, button'
    }
  },

  setup(props) {
    let animation;
    const target = {
      x: 0,
      y: 0
    };
    const cursors = [];
    const cursorEl = ref();
    const mouseOut = ref(true),
          clicking = ref(false),
          hoverActive = ref(false);

    function draw() {
      cursors.forEach(({
        el,
        p
      }) => followMouse(el, p, target));
      animation = requestAnimationFrame(draw);
    }

    function handleMouseMove(e) {
      const el = e.target;
      hoverActive.value = !!(el !== null && el !== void 0 && el.matches(props.hover));
      mouseOut.value = false;
      target.x = e.clientX - props.size / 2;
      target.y = e.clientY - props.size / 2;
    }

    onMounted(() => {
      var _cursorEl$value;

      // Set variables: (cursors element list)
      // And set "--scale" css variable
      const children = (_cursorEl$value = cursorEl.value) === null || _cursorEl$value === void 0 ? void 0 : _cursorEl$value.children;

      if (children) {
        [...children].forEach((el, i) => {
          const p = valToP(i + 1, 0, props.length);
          cursors.push({
            el,
            p
          });
          el.style.setProperty('--scale', String(p));
        });
      }

      animation = requestAnimationFrame(draw);
    });
    onUnmounted(() => {
      cancelAnimationFrame(animation);
    });
    return {
      cursorEl,
      handleMouseMove,
      mouseOut,
      clicking,
      hoverActive
    };
  }

});

const _withId = /*#__PURE__*/withScopeId("data-v-3798d71c");

pushScopeId("data-v-3798d71c");

const _hoisted_1 = /*#__PURE__*/createVNode("svg", {
  class: "svg-effect"
}, [/*#__PURE__*/createVNode("defs", null, [/*#__PURE__*/createVNode("filter", {
  id: "gooey-cursor-svg-effect"
}, [/*#__PURE__*/createVNode("feGaussianBlur", {
  in: "SourceGraphic",
  stdDeviation: "6",
  result: "blur"
}), /*#__PURE__*/createVNode("feColorMatrix", {
  in: "blur",
  mode: "matrix",
  values: "1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 35 -15",
  result: "gooey-cursor-svg-effect"
}), /*#__PURE__*/createVNode("feComposite", {
  in: "SourceGraphic",
  in2: "gooey-cursor-svg-effect",
  operator: "atop"
})])])], -1);

const _hoisted_2 = /*#__PURE__*/createVNode("svg", {
  viewBox: "0 0 10 10"
}, [/*#__PURE__*/createVNode("circle", {
  cx: "5",
  cy: "5",
  r: "4"
})], -1);

popScopeId();

const render = /*#__PURE__*/_withId((_ctx, _cache, $props, $setup, $data, $options) => {
  const _component_GlobalEvents = resolveComponent("GlobalEvents");

  return openBlock(), createBlock(Fragment, null, [_hoisted_1, createVNode("div", {
    ref: "cursorEl",
    class: ["gooey-cursor", {
      mouseOut: _ctx.mouseOut,
      clicking: _ctx.clicking,
      hoverActive: _ctx.hoverActive
    }],
    style: // @ts-ignore
    {
      '--color': _ctx.color || '',
      '--size': _ctx.size ? `${_ctx.size}px` : ''
    }
  }, [(openBlock(true), createBlock(Fragment, null, renderList(_ctx.length, i => {
    return openBlock(), createBlock("div", {
      key: i
    }, [_hoisted_2]);
  }), 128))], 6), createVNode(_component_GlobalEvents, {
    onMousemove: _ctx.handleMouseMove,
    onMouseout: _cache[1] || (_cache[1] = $event => _ctx.mouseOut = true),
    onMouseleave: _cache[2] || (_cache[2] = $event => _ctx.mouseOut = true),
    onMouseenter: _cache[3] || (_cache[3] = $event => _ctx.mouseOut = false),
    onMousedown: _cache[4] || (_cache[4] = $event => _ctx.clicking = true),
    onMouseup: _cache[5] || (_cache[5] = $event => _ctx.clicking = false)
  }, null, 8, ["onMousemove"])], 64);
});

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".svg-effect[data-v-3798d71c] {\n  position: absolute;\n  width: 0;\n  height: 0;\n  display: none;\n  pointer-events: none;\n}\n.gooey-cursor[data-v-3798d71c] {\n  pointer-events: none !important;\n  position: fixed !important;\n  z-index: var(--z-index, 9999);\n  top: 0 !important;\n  left: 0 !important;\n  margin: 0 !important;\n  padding: 0 !important;\n  filter: url(\"#gooey-cursor-svg-effect\");\n  width: var(--size, 2rem);\n  height: var(--size, 2rem);\n  --largest-scale: 1;\n  transition: opacity 0.2s;\n}\n.gooey-cursor div[data-v-3798d71c] {\n  --scale: 1;\n  position: absolute;\n  display: flex;\n  box-sizing: border-box;\n  top: 0;\n  left: 0;\n  width: 100%;\n  height: 100%;\n  will-change: transform;\n}\n.gooey-cursor div svg[data-v-3798d71c] {\n  margin: auto;\n  width: 100%;\n  height: 100%;\n  transform: scale(var(--scale));\n}\n.gooey-cursor div svg circle[data-v-3798d71c] {\n  fill: var(--color, #42b883);\n}\n.gooey-cursor div:last-of-type svg[data-v-3798d71c] {\n  transform: scale(var(--largest-scale));\n  transition: transform 100ms cubic-bezier(0.51, 0.06, 0.56, 1.37);\n}\n.gooey-cursor.mouseOut[data-v-3798d71c] {\n  opacity: 0;\n}\n.gooey-cursor.hoverActive[data-v-3798d71c] {\n  opacity: 0.6;\n  --largest-scale: 1.4;\n}\n.gooey-cursor.hoverActive.clicking[data-v-3798d71c] {\n  --largest-scale: 1.1;\n}\n.gooey-cursor.clicking[data-v-3798d71c] {\n  --largest-scale: 1.3;\n}";
styleInject(css_248z);

script.render = render;
script.__scopeId = "data-v-3798d71c";

// Import vue component
// IIFE injects install function into component, allowing component
// to be registered via Vue.use() as well as Vue.component(),

var entry_esm = /*#__PURE__*/(() => {
  // Assign InstallableComponent type
  const installable = script; // Attach install function executed by Vue.use()

  installable.install = app => {
    app.component('VueGooeyCursor', installable);
  };

  return installable;
})(); // It's possible to expose named exports when writing components that can
// also be used as directives, etc. - eg. import { RollupDemoDirective } from 'rollup-demo';
// export const RollupDemoDirective = directive;

export { entry_esm as default };
